@@ -4,7 +4,7 @@ use romu::Rng;
 pub fn scalar(c: &mut Criterion) {
     let mut group = c.benchmark_group("scalar");
 
-    let rng = Rng::new().unwrap();
+    let rng = Rng::new();
 
     group.bench_function(BenchmarkId::new("u8", ""), |b| {
         b.iter(|| {
@@ -87,10 +87,48 @@ pub fn scalar(c: &mut Criterion) {
     group.finish();
 }
 
+pub fn mod_u(c: &mut Criterion) {
+    let mut group = c.benchmark_group("mod");
+
+    let rng = Rng::new();
+
+    group.bench_function(BenchmarkId::new("u8", ""), |b| {
+        b.iter(|| {
+            black_box(rng.mod_u8(u8::MAX - 1));
+        })
+    });
+
+    group.bench_function(BenchmarkId::new("u16", ""), |b| {
+        b.iter(|| {
+            black_box(rng.mod_u16(u16::MAX - 1));
+        })
+    });
+
+    group.bench_function(BenchmarkId::new("u32", ""), |b| {
+        b.iter(|| {
+            black_box(rng.mod_u32(u32::MAX - 1));
+        })
+    });
+
+    group.bench_function(BenchmarkId::new("u64", ""), |b| {
+        b.iter(|| {
+            black_box(rng.mod_u64(u64::MAX - 1));
+        })
+    });
+
+    group.bench_function(BenchmarkId::new("usize", ""), |b| {
+        b.iter(|| {
+            black_box(rng.mod_usize(usize::MAX - 1));
+        })
+    });
+
+    group.finish();
+}
+
 pub fn range(c: &mut Criterion) {
     let mut group = c.benchmark_group("range");
 
-    let rng = Rng::new().unwrap();
+    let rng = Rng::new();
 
     group.bench_function(BenchmarkId::new("u8", ""), |b| {
         b.iter(|| {
@@ -158,20 +196,13 @@ pub fn range(c: &mut Criterion) {
 pub fn bytes(c: &mut Criterion) {
     let mut group = c.benchmark_group("bytes");
 
-    let rng = Rng::new().unwrap();
+    let rng = Rng::new();
 
     let size = 1024 * 1024; // 1 MiB
     group.throughput(Throughput::Bytes(size as u64));
 
     let mut buffer = vec![0u8; size];
 
-    group.bench_with_input(BenchmarkId::new("Copy", size), &size, |b, &_s| {
-        b.iter(|| fill_dummy_bytes(&mut buffer));
-    });
-    black_box(buffer);
-
-    let mut buffer = vec![0u8; size];
-
     group.bench_with_input(BenchmarkId::new("Rng", size), &size, |b, &_s| {
         b.iter(|| rng.fill_bytes(&mut buffer));
     });
@@ -181,7 +212,7 @@ pub fn bytes(c: &mut Criterion) {
     {
         use romu::{Rng128, Rng256, Rng512};
 
-        let rng = Rng128::new().unwrap();
+        let rng = Rng128::new();
 
         let mut buffer = vec![0u8; size];
 
@@ -190,7 +221,7 @@ pub fn bytes(c: &mut Criterion) {
         });
         black_box(buffer);
 
-        let rng = Rng256::new().unwrap();
+        let rng = Rng256::new();
 
         let mut buffer = vec![0u8; size];
 
@@ -199,7 +230,7 @@ pub fn bytes(c: &mut Criterion) {
         });
         black_box(buffer);
 
-        let rng = Rng512::new().unwrap();
+        let rng = Rng512::new();
 
         let mut buffer = vec![0u8; size];
 
@@ -216,8 +247,8 @@ pub fn bytes(c: &mut Criterion) {
 pub fn tls(c: &mut Criterion) {
     let mut group = c.benchmark_group("tls");
 
-    let rng = Rng::new().unwrap();
-    romu::seed().unwrap();
+    let rng = Rng::new();
+    romu::seed();
 
     group.bench_function(BenchmarkId::new("u64", "instanced"), |b| {
         b.iter(|| {
@@ -231,8 +262,8 @@ pub fn tls(c: &mut Criterion) {
         })
     });
 
-    let rng = Rng::new().unwrap();
-    romu::seed().unwrap();
+    let rng = Rng::new();
+    romu::seed();
 
     let count = 1024 * 1024;
     group.throughput(Throughput::Bytes(count));
@@ -254,22 +285,9 @@ pub fn tls(c: &mut Criterion) {
     group.finish();
 }
 
-fn fill_dummy_bytes(slice: &mut [u8]) {
-    let data = [0u8; 32];
-    let mut chunks = slice.chunks_exact_mut(32);
-    for chunk in &mut chunks {
-        chunk.copy_from_slice(&data)
-    }
-    chunks
-        .into_remainder()
-        .iter_mut()
-        .enumerate()
-        .for_each(|(i, x)| *x = data[i]);
-}
-
 #[cfg(feature = "tls")]
-criterion_group!(benches, scalar, range, bytes, tls);
+criterion_group!(benches, scalar, mod_u, range, bytes, tls);
 #[cfg(not(feature = "tls"))]
-criterion_group!(benches, scalar, range, bytes);
+criterion_group!(benches, scalar, mod_u, range, bytes);
 
 criterion_main!(benches);
