@@ -1,17 +1,23 @@
 use core::ops::RangeBounds;
 
-use crate::{Rng, SeedError};
+use crate::{Rng, SeedSource};
 
 thread_local! {
-    static RNG: Rng = Rng::from_seed_with_192bit([0x50b88b1357314d73, 0x8e4193b54c35742c, 0x47c6ff4737815fc5]);
+    static RNG: Rng = Rng::fixed_tls();
 }
 
 #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
-/// Seeds the thread local instance from the best available entropy pool.
-pub fn seed() -> Result<(), SeedError> {
+/// Seeds the thread local instance from the best available randomness source.
+pub fn seed() {
     RNG.with(|f| f.seed())
 }
 
+#[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
+/// Shows which source was used to acquire the seed for the thread local instance.
+pub fn seed_source() -> SeedSource {
+    RNG.with(|f| f.seed_source())
+}
+
 #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
 /// Seeds the thread local instance with the given 64-bit seed.
 pub fn seed_with_64bit(seed: u64) {
@@ -36,7 +42,7 @@ pub fn seed_with_192bit(seed: [u64; 3]) {
 #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
 /// Mixes the state, which should improve the quality of the random numbers.
 ///
-/// Should be called when having (re-)seeded the generator with a fixed value of low entropy.
+/// Should be called when having (re-)seeded the generator with a fixed value of low randomness.
 pub fn mix() {
     RNG.with(|rng| rng.mix())
 }
@@ -112,14 +118,14 @@ pub fn isize() -> isize {
 }
 
 #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
-/// Generates a random f32 value.
+/// Generates a random f32 value in range (0..1).
 #[inline(always)]
 pub fn f32() -> f32 {
     RNG.with(|rng| rng.f32())
 }
 
 #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
-/// Generates a random f64 value.
+/// Generates a random f64 value in range (0..1).
 #[inline(always)]
 pub fn f64() -> f64 {
     RNG.with(|rng| rng.f64())
@@ -196,60 +202,70 @@ pub fn mod_usize(n: usize) -> usize {
 
 #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
 /// Generates a random u8 value in the given range.
+#[inline(always)]
 pub fn range_u8<T: RangeBounds<u8>>(range: T) -> u8 {
     RNG.with(|rng| rng.range_u8(range))
 }
 
 #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
 /// Generates a random u16 value in the given range.
+#[inline(always)]
 pub fn range_u16<T: RangeBounds<u16>>(range: T) -> u16 {
     RNG.with(|rng| rng.range_u16(range))
 }
 
 #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
 /// Generates a random u32 value in the given range.
+#[inline(always)]
 pub fn range_u32<T: RangeBounds<u32>>(range: T) -> u32 {
     RNG.with(|rng| rng.range_u32(range))
 }
 
 #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
 /// Generates a random u64 value in the given range.
+#[inline(always)]
 pub fn range_u64<T: RangeBounds<u64>>(range: T) -> u64 {
     RNG.with(|rng| rng.range_u64(range))
 }
 
 #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
 /// Generates a random usize value in the given range.
+#[inline(always)]
 pub fn range_usize<T: RangeBounds<usize>>(range: T) -> usize {
     RNG.with(|rng| rng.range_usize(range))
 }
 
 #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
 /// Generates a random i8 value in the given range.
+#[inline(always)]
 pub fn range_i8<T: RangeBounds<i8>>(range: T) -> i8 {
     RNG.with(|rng| rng.range_i8(range))
 }
 
 #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
 /// Generates a random i16 value in the given range.
+#[inline(always)]
 pub fn range_i16<T: RangeBounds<i16>>(range: T) -> i16 {
     RNG.with(|rng| rng.range_i16(range))
 }
 
 #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
 /// Generates a random i32 value in the given range.
+#[inline(always)]
 pub fn range_i32<T: RangeBounds<i32>>(range: T) -> i32 {
     RNG.with(|rng| rng.range_i32(range))
 }
 
 #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
 /// Generates a random i64 value in the given range.
+#[inline(always)]
 pub fn range_i64<T: RangeBounds<i64>>(range: T) -> i64 {
     RNG.with(|rng| rng.range_i64(range))
 }
 
 #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
 /// Generates a random isize value in the given range.
+#[inline(always)]
 pub fn range_isize<T: RangeBounds<isize>>(range: T) -> isize {
     RNG.with(|rng| rng.range_isize(range))
 }
